@@ -1,12 +1,12 @@
 package Net::LDAP::Express;
 
-# $Id: Express.pm,v 1.4 2003/05/18 22:22:54 bronto Exp $
+# $Id: Express.pm,v 1.6 2004/12/04 18:10:20 bronto Exp $
 
 use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.10';
 
 use Carp ;
 
@@ -36,8 +36,8 @@ sub new {
 
   # Test for onlyattrs not overlapping with searchextras; in
   # case they do, warn
-  if (exists $localparms{onlyattrs} and
-      exists $localparms{searchextras}) {
+  if (defined $localparms{onlyattrs} and
+      defined $localparms{searchextras}) {
     carp "Useless use of parameter onlyattrs with searchextras" if $^W ;
   }
 
@@ -89,7 +89,7 @@ sub new {
     $lasterr[0] = shift ;
 
     # Update cache, if needed
-    $errcache{$lasterr[0]} ||= shift ;
+    $errcache{$lasterr[0]} = shift unless exists $errcache{$lasterr[0]} ;
 
     # Get error message from cache
     $lasterr[1] = $errcache{$lasterr[0]} ;
@@ -288,29 +288,24 @@ sub _makefilter {
   my ($query) = @_ ;
 
   my $bool  = $ldap->_searchbool  ? $ldap->_searchbool  : '|'  ;
+  my $match = $ldap->_searchmatch ;
 
   my $op    = '~=' ;
-  $op = '=' if $ldap->_searchmatch and $ldap->_searchmatch eq 'substr' ;
+
+  if ($match) {
+    $op = '=' if $match eq 'substr' or $match eq 'exact' ;
+    $query = qq/*$query*/ if $match eq 'substr';
+  }
 
   my @attrs = @{$ldap->_searchattrs} ;
 
-  $query    = qq/*$query*/ if $op eq '=' ;
-
   my $filter ;
   if (@attrs == 1) {
     $filter = qq/($attrs[0]$op$query)/ ;
-  }
-
-  if (@attrs == 2) {
-    $filter = qq/($bool($attrs[0]$op$query)($attrs[1]$op$query))/ ;
-  }
-
-  if (@attrs > 2) {
-    $filter = qq/($bool($attrs[0]$op$query)($attrs[1]$op$query))/ ;
-    splice @attrs,0,2 ;
-    while (my $attr = shift @attrs) {
-      $filter = qq/($bool($attr$op$query)$filter)/ ;
-    }
+  } else {
+    $filter = "($bool".
+              join("",map("($_$op$query)",@attrs)).
+	      ")" ;
   }
 
   #carp "Search filter is $filter" if DEBUG ;
@@ -364,6 +359,16 @@ __END__
 
 Net::LDAP::Express - Simplified interface for Net::LDAP
 
+=head1 WARNING
+
+With version 0.10 the return value for the C<error> method has slightly
+changed; on no-error condition (error code 0) it returns a null string,
+and not C<undef> any more. Code that simply checked for boolean true/false
+value will continue to work as expected, but code that relied on C<undef>
+and the C<defined> function won't!
+B<Please let me know if this behaviour breaks
+any existing application!>. Thanks.
+
 =head1 SYNOPSIS
 
   use Net::LDAP::Express;
@@ -510,6 +515,11 @@ parameters are therefore:
 the name or IP address of the directory server we are connecting
 to. Mandatory.
 
+=item port
+
+the port to connect to; if omitted, the 389 will be used. 389 is the
+LDAP standard port.
+
 =item bindDN
 
 bind DN in case of authenticated bind
@@ -537,7 +547,9 @@ operators are | and &.
 By default, an 'approx' search is performed by simplesearch(); for
 those directory servers that doesn't support the ~= operator it is
 possible to request a substring search specifying the value 'substr'
-for the searchmatch parameter.
+for the searchmatch parameter.  Alternatively, if this is set to 'exact'
+then an exact search will be done - useful when fields are not indexed
+for substring searching.
 
 =item searchextras
 
@@ -626,11 +638,17 @@ update takes a list of Net::LDAP::Entry objects as arguments and
 commits changes on the directory server. Returns a reference to an
 array of updated entries.
 
+B<NOTE:> if you want to modify an entry, say C<$e>, remember to call
+C<$e-E<gt>changetype('modify')> on it B<before> doing any changes; the
+defined changetype at object creation is C<add> at the moment, which
+results in C<update> trying to create new entries. This could be
+addressed by Net::LDAP::Express in the future, maybe.
+
 =item simplesearch($searchstring)
 
 Searches entries using the new()'s search* and base parameters. Takes
-a search string as argument. Returns a list of entries on success,
-undef on error.
+a search string as argument. Returns a reference to an array of
+entries on success, undef on error.
 
 =item error
 
@@ -647,6 +665,9 @@ Returns last error's code
 
 Marco Marongiu, E<lt>bronto@cpan.orgE<gt>
 
+Original patch for exact matching (code and documentation) was
+kindly contributed by Gordon Lack.
+
 =head1 SEE ALSO
 
 L<Net::LDAP>.
